use crate::error::*;
use bb8_postgres::tokio_postgres::NoTls;
use bb8_postgres::PostgresConnectionManager;
use std::time::Duration;

pub enum Pool {
    Postgres(bb8::Pool<PostgresConnectionManager<NoTls>>),
}

pub enum Client<'a> {
    Postgres(bb8::PooledConnection<'a, PostgresConnectionManager<NoTls>>),
}

static mut POOL: Option<Pool> = None;

pub async fn init() -> Result<(), Error> {
    let manager = PostgresConnectionManager::new(
        "host=localhost user=postgres dbname=locutus"
            .parse()
            .unwrap(),
        NoTls,
    );

    let pool = bb8::Pool::builder()
        .max_size(64)
        .min_idle(Some(2))
        .connection_timeout(Duration::from_secs(4))
        .max_lifetime(Some(Duration::from_secs(3600)))
        .build(manager)
        .await?;

    unsafe {
        POOL = Some(Pool::Postgres(pool));
    }

    Ok(())
}

pub async fn client() -> Result<Client<'static>, Error> {
    match unsafe { POOL.as_ref().unwrap().clone() } {
        Pool::Postgres(pool) => Ok(Client::Postgres(pool.get().await?)),
    }
}
