mod authenticate;
mod new_event;
mod ping;
mod subscribe;

use crate::usimp::*;
use tokio::sync::mpsc;
use crate::websocket::WebSocketEnvelope;

pub async fn endpoint(
    input: &InputEnvelope,
    tx: Option<mpsc::Sender<WebSocketEnvelope>>,
) -> Result<OutputEnvelope, Error> {
    if input.from_domain != None {
        // TODO
        return Err(Error::new(
            ErrorKind::NotImplemented,
            ErrorClass::ServerError,
            None,
        ));
    }
    let session;
    if let Some(token) = &input.token {
        session = Some(Session::from_token(token).await?);
    } else {
        session = None;
    }

    println!("Endpoint: {}", input.endpoint);
    Ok(match input.endpoint.as_str() {
        "ping" => input.respond(ping::handle(&input, session).await?),
        "authenticate" => input.respond(authenticate::handle(&input, session).await?),
        "subscribe" => input.respond(subscribe::handle(&input, session, tx).await?),
        "new_event" => input.respond(new_event::handle(&input, session).await?),
        _ => input.new_error(
            ErrorKind::UsimpError,
            ErrorClass::ClientProtocolError,
            Some("Invalid endpoint".to_string()),
        ),
    })
}
