use crate::usimp::subscription;
use crate::usimp::*;

use serde::{Deserialize, Serialize};
use serde_json::{from_value, to_value, Value};
use tokio::sync::mpsc;
use crate::websocket::WebSocketEnvelope;

#[derive(Serialize, Deserialize, Clone)]
struct Input {}

#[derive(Serialize, Deserialize, Clone)]
struct Output {
    events: Vec<Event>,
}

pub async fn handle(
    input: &InputEnvelope,
    session: Option<Session>,
    tx: Option<mpsc::Sender<WebSocketEnvelope>>,
) -> Result<Value, Error> {
    Ok(to_value(
        subscribe(
            from_value(input.data.clone())?,
            session,
            input.request_nr,
            tx,
        )
        .await?,
    )?)
}

async fn subscribe(
    _input: Input,
    session: Option<Session>,
    req_nr: Option<u64>,
    tx: Option<mpsc::Sender<WebSocketEnvelope>>,
) -> Result<Output, Error> {
    let account = get_account(&session)?;
    let mut rx = subscription::subscribe_account(account).await;
    match tx {
        Some(tx) => {
            tokio::spawn(async move {
                while let Some(event) = rx.recv().await {
                    let _res = tx
                        .send(OutputEnvelope {
                            error: None,
                            request_nr: req_nr,
                            data: serde_json::json![{"events": [event]}],
                        }.into())
                        .await;
                }
            });
            Ok(Output { events: vec![] })
        }
        None => {
            if let Some(event) = rx.recv().await {
                Ok(Output { events: vec![event] })
            } else {
                Err(Error::new(
                    ErrorKind::SubscriptionError,
                    ErrorClass::ServerError,
                    None,
                ))
            }
        }
    }
}
