use crate::database;
use crate::usimp::*;
use std::collections::HashMap;
use std::sync::Arc;
use tokio::sync::{mpsc, Mutex};

static mut ROOMS: Option<Arc<Mutex<HashMap<Uuid, Vec<mpsc::Sender<Event>>>>>> = None;
static mut ACCOUNTS: Option<Arc<Mutex<HashMap<Uuid, Vec<mpsc::Sender<Event>>>>>> = None;

pub fn init() {
    unsafe {
        ROOMS = Some(Arc::new(Mutex::new(HashMap::new())));
        ACCOUNTS = Some(Arc::new(Mutex::new(HashMap::new())));
    }
}

pub async fn subscribe_account(account: &Account) -> mpsc::Receiver<Event> {
    let (tx, rx) = mpsc::channel::<Event>(64);
    unsafe {
        let mut acc = ACCOUNTS.as_ref().unwrap().lock().await;
        match acc.get_mut(&account.id) {
            Some(vec) => {
                vec.push(tx);
            }
            None => {
                acc.insert(account.id, vec![tx]);
            }
        }
    }
    rx
}

pub async fn push(room_id: &Uuid, event: Event) -> Result<(), Error> {
    let backend = database::client().await?;
    let accounts = match backend {
        database::Client::Postgres(client) => {
            let res = client
                .query(
                    "SELECT account_id \
                    FROM member \
                    WHERE room_id = $1;",
                    &[&room_id],
                )
                .await?;
            let mut acc: Vec<Uuid> = Vec::new();
            for row in res {
                acc.push(row.get(0));
            }
            acc
        }
    };

    let mut rooms = unsafe {
        let mut rooms = ROOMS.as_ref().unwrap().lock().await;
        if let Some(rooms) = rooms.get_mut(room_id) {
            rooms.clone()
        } else {
            Vec::new()
        }
    };

    for account in accounts {
        unsafe {
            let mut accounts = ACCOUNTS.as_ref().unwrap().lock().await;
            if let Some(acc) = accounts.get_mut(&account) {
                let mut acc = acc.clone();
                rooms.append(&mut acc);
            }
        }
    }

    for tx in rooms {
        let _res = tx.send(event.clone()).await;
    }

    Ok(())
}
